@@ -52,28 +52,33 @@ typedef struct _GstControlPoint GstControlPoint;
 
 /**
  * GstControlPoint:
+ * @timestamp: timestamp of the value change
+ * @value: the new value
  *
- * a internal structure for value+time and various temporary
+ * An internal structure for value+time and various temporary
  * values used for interpolation. This "inherits" from
  * GstTimedValue.
  */
 struct _GstControlPoint
 {
   /* fields from GstTimedValue. DO NOT CHANGE! */
-  GstClockTime timestamp;       /* timestamp of the value change */
-  gdouble value;                /* the new value */
+  GstClockTime timestamp;
+  gdouble value;
 
-  /* internal fields */
+  /*< private >*/
 
   /* Caches for the interpolators */
   /* FIXME: we should not have this here already ... */
   union {
-    struct {
+    struct { /* 16 bytes */
       gdouble h;
       gdouble z;
     } cubic;
+    struct { /* 24 bytes */
+      gdouble c1s, c2s, c3s;
+    } cubic_mono;
+    guint8 _gst_reserved[64];
   } cache;
-
 };
 
 GType gst_control_point_get_type (void);
@@ -130,6 +135,10 @@ GList *         gst_timed_value_control_source_get_all        (GstTimedValueCont
 gint            gst_timed_value_control_source_get_count      (GstTimedValueControlSource * self);
 void            gst_timed_value_control_invalidate_cache      (GstTimedValueControlSource * self);
 
+#ifdef G_DEFINE_AUTOPTR_CLEANUP_FUNC
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstTimedValueControlSource, gst_object_unref)
+#endif
+
 G_END_DECLS
 
 #endif /* __GST_TIMED_VALUE_CONTROL_SOURCE_H__ */
