@@ -143,6 +143,10 @@ void            gst_ptp_statistics_callback_remove (gulong id);
 GstClock*       gst_ptp_clock_new                  (const gchar *name,
                                                     guint domain);
 
+#ifdef G_DEFINE_AUTOPTR_CLEANUP_FUNC
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstPtpClock, gst_object_unref)
+#endif
+
 G_END_DECLS
 
 #endif /* __GST_PTP_CLOCK_H__ */
