@@ -115,6 +115,7 @@ struct _GstQueue2
   /* for measuring input/output rates */
   GTimer *in_timer;
   gboolean in_timer_started;
+  gdouble last_update_in_rates_elapsed;
   gdouble last_in_elapsed;
   guint64 bytes_in;
   gdouble byte_in_rate;
@@ -165,6 +166,9 @@ struct _GstQueue2
 struct _GstQueue2Class
 {
   GstElementClass parent_class;
+
+  /* signals */
+  void (*overrun)       (GstQueue2 *queue2);
 };
 
 G_GNUC_INTERNAL GType gst_queue2_get_type (void);
