@@ -361,8 +361,7 @@ gst_tee_request_new_pad (GstElement * element, GstPadTemplate * templ,
 
   GST_OBJECT_LOCK (tee);
 
-  if (name_templ) {
-    sscanf (name_templ, "src_%u", &index);
+  if (name_templ && sscanf (name_templ, "src_%u", &index) == 1) {
     GST_LOG_OBJECT (element, "name: %s (index %d)", name_templ, index);
     if (g_hash_table_contains (tee->pad_indexes, GUINT_TO_POINTER (index))) {
       GST_ERROR_OBJECT (element, "pad name %s is not unique", name_templ);
